@@ -278,15 +278,81 @@ bool Serial::setParams(int speed, Parity parity, int bits, int stop_bits,
     case 38400:
       serial_speed = B38400;
       break;
+#ifdef B57600
     case 57600:
       serial_speed = B57600;
       break;
+#endif
+#ifdef B115200
     case 115200:
       serial_speed = B115200;
       break;
+#endif
+#ifdef B230400
     case 230400:
       serial_speed = B230400;
       break;
+#endif
+#ifdef B460800
+    case 460800:
+      serial_speed = B460800;
+      break;
+#endif
+#ifdef B500000
+    case 500000:
+      serial_speed = B500000;
+      break;
+#endif
+#ifdef B576000
+    case 576000:
+      serial_speed = B576000;
+      break;
+#endif
+#ifdef B921600
+    case 921600:
+      serial_speed = B921600;
+      break;
+#endif
+#ifdef B1000000
+    case 1000000:
+      serial_speed = B1000000;
+      break;
+#endif
+#ifdef B1152000
+    case 1152000:
+      serial_speed = B1152000;
+      break;
+#endif
+#ifdef B1500000
+    case 1500000:
+      serial_speed = B1500000;
+      break;
+#endif
+#ifdef B2000000
+    case 2000000:
+      serial_speed = B2000000;
+      break;
+#endif
+#ifdef B2500000
+    case 2500000:
+      serial_speed = B2500000;
+      break;
+#endif
+#ifdef B3000000
+    case 3000000:
+      serial_speed = B3000000;
+      break;
+#endif
+#ifdef B3500000
+    case 3500000:
+      serial_speed = B3500000;
+      break;
+#endif
+#ifdef B4000000
+    case 4000000:
+      serial_speed = B4000000;
+      break;
+#endif
     default:
       errno = EINVAL;
       return false;
