@@ -635,6 +635,12 @@ int Directory::handleCallList(char *buf, int len)
 	  get_call_list.clear();
 	  com_state = CS_IDLE;
 	  read_len = 3;
+
+          std::string errmsg("INCORRECT PASSWORD");
+          if (the_message.find(errmsg.c_str(), 0, errmsg.size()) == 0)
+          {
+            error_str = the_message;
+          }
 	}
 	else
 	{
@@ -688,9 +694,9 @@ void Directory::ctrlSockConnected(void)
     case Cmd::ONLINE:
     {
       time_t t = time(NULL);
-      struct tm *tm = localtime(&t);
+      struct tm tm;
       char local_time_str[6];
-      strftime(local_time_str, 6, "%H:%M", tm);
+      strftime(local_time_str, 6, "%H:%M", localtime_r(&t, &tm));
       cmdstr = "l" + the_callsign + "\254\254" + the_password +
 	  "\015ONLINE3.38(" + local_time_str + ")\015" + the_description +
 	  "\015";
@@ -700,9 +706,9 @@ void Directory::ctrlSockConnected(void)
     case Cmd::BUSY:
     {
       time_t t = time(NULL);
-      struct tm *tm = localtime(&t);
+      struct tm tm;
       char local_time_str[6];
-      strftime(local_time_str, 6, "%H:%M", tm);
+      strftime(local_time_str, 6, "%H:%M", localtime_r(&t, &tm));
       cmdstr = "l" + the_callsign + "\254\254" + the_password +
 	  "\015BUSY3.40(" + local_time_str + ")\015" + the_description + "\015";
       break;
