@@ -10,7 +10,7 @@ specific logic core classes (e.g. SimplexLogic and RepeaterLogic).
 
 \verbatim
 SvxLink - A Multi Purpose Voice Services System for Ham Radio Use
-Copyright (C) 2003-2018 Tobias Blomberg / SM0SVX
+Copyright (C) 2003-2022 Tobias Blomberg / SM0SVX
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -89,7 +89,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "LogicCmds.h"
 #include "Logic.h"
 #include "QsoRecorder.h"
-#include "LinkManager.h"
+//#include "LinkManager.h"
 #include "DtmfDigitHandler.h"
 
 
@@ -152,9 +152,8 @@ using namespace SvxLink;
  *
  ****************************************************************************/
 
-Logic::Logic(Config &cfg, const string& name)
-  : LogicBase(cfg, name),
-    m_rx(0),  	      	      	            m_tx(0),
+Logic::Logic(void)
+  : m_rx(0),  	      	      	            m_tx(0),
     msg_handler(0), 	      	            active_module(0),
     exec_cmd_on_sql_close_timer(-1),        rgr_sound_timer(-1),
     report_ctcss(0.0f),                     event_handler(0),
@@ -170,7 +169,8 @@ Logic::Logic(Config &cfg, const string& name)
     tx_ctcss_mask(0),
     currently_set_tx_ctrl_mode(Tx::TX_OFF), is_online(true),
     dtmf_digit_handler(0),                  state_pty(0),
-    dtmf_ctrl_pty(0)
+    dtmf_ctrl_pty(0),                       command_pty(0),
+    m_ctcss_to_tg_timer(-1),                m_ctcss_to_tg_last_fq(0.0f)
 {
   rgr_sound_timer.expired.connect(sigc::hide(
         mem_fun(*this, &Logic::sendRgrSound)));
@@ -179,18 +179,9 @@ Logic::Logic(Config &cfg, const string& name)
 } /* Logic::Logic */
 
 
-Logic::~Logic(void)
+bool Logic::initialize(Async::Config& cfgobj, const std::string& logic_name)
 {
-  cleanup();
-  delete logic_con_out;
-  delete logic_con_in;
-  delete dtmf_digit_handler;
-} /* Logic::~Logic */
-
-
-bool Logic::initialize(void)
-{
-  if (!LogicBase::initialize())
+  if (!LogicBase::initialize(cfgobj, logic_name))
   {
     return false;
   }
@@ -207,6 +198,8 @@ bool Logic::initialize(void)
     }
   }
 
+  cfg().getValue(name(), "ONLINE", is_online);
+
   ChangeLangCmd *lang_cmd = new ChangeLangCmd(&cmd_parser, this);
   if (!lang_cmd->addToParser())
   {
@@ -292,6 +285,24 @@ bool Logic::initialize(void)
         mem_fun(*this, &Logic::dtmfCtrlPtyCmdReceived));
   }
 
+  string command_pty_path;
+  cfg().getValue(name(), "COMMAND_PTY", command_pty_path);
+  if (!command_pty_path.empty())
+  {
+    command_pty = new Pty(command_pty_path);
+    if (!command_pty->open())
+    {
+      cerr << "*** ERROR: Could not open command PTY "
+           << command_pty_path << " as specified in configuration variable "
+           << name() << "/" << "COMMAND_PTY" << endl;
+      cleanup();
+      return false;
+    }
+    command_pty->setLineBuffered(true);
+    command_pty->dataReceived.connect(
+        mem_fun(*this, &Logic::commandPtyCmdReceived));
+  }
+
   string value;
   if (cfg().getValue(name(), "ACTIVATE_MODULE_ON_LONG_CMD", value))
   {
@@ -384,7 +395,7 @@ bool Logic::initialize(void)
   AudioSource *prev_rx_src = 0;
 
     // Create the RX object
-  cout << "Loading RX: " << rx_name << endl;
+  cout << name() << ": Loading RX \"" << rx_name << "\"" << endl;
   m_rx = RxFactory::createNamedRx(cfg(), rx_name);
   if ((m_rx == 0) || !rx().initialize())
   {
@@ -394,12 +405,20 @@ bool Logic::initialize(void)
     cleanup();
     return false;
   }
-  rx().squelchOpen.connect(mem_fun(*this, &Logic::squelchOpen));
+  rx().squelchOpen.connect([&](bool is_open) {
+        if (!is_open)
+        {
+          //std::cout << "### Logic::[]: Disable CTCSS to TG timer"
+          //          << std::endl;
+          m_ctcss_to_tg_timer.setEnable(false);
+        }
+        squelchOpen(is_open);
+      });
   rx().dtmfDigitDetected.connect(mem_fun(*this, &Logic::dtmfDigitDetectedP));
   rx().selcallSequenceDetected.connect(
 	mem_fun(*this, &Logic::selcallSequenceDetected));
   rx().setMuteState(Rx::MUTE_NONE);
-  rx().publishStateEvent.connect(mem_fun(*this, &Logic::publishStateEvent));
+  rx().publishStateEvent.connect(mem_fun(*this, &Logic::onPublishStateEvent));
   prev_rx_src = m_rx;
 
     // This valve is used to turn RX audio on/off into the logic core
@@ -545,7 +564,7 @@ bool Logic::initialize(void)
   prev_tx_src = tx_audio_mixer;
 
     // Create the TX object
-  cout << "Loading TX: " << tx_name << endl;
+  std::cout << name() << ": Loading TX \"" << tx_name << "\"" << endl;
   m_tx = TxFactory::createNamedTx(cfg(), tx_name);
   if ((m_tx == 0) || !tx().initialize())
   {
@@ -557,6 +576,7 @@ bool Logic::initialize(void)
   }
   tx().transmitterStateChange.connect(
       mem_fun(*this, &Logic::transmitterStateChange));
+  tx().publishStateEvent.connect(mem_fun(*this, &Logic::onPublishStateEvent));
   prev_tx_src->registerSink(m_tx);
   prev_tx_src = 0;
 
@@ -588,9 +608,11 @@ bool Logic::initialize(void)
   event_handler->deactivateModule.connect(
           bind(mem_fun(*this, &Logic::deactivateModule), (Module *)0));
   event_handler->publishStateEvent.connect(
-          mem_fun(*this, &Logic::publishStateEvent));
+          mem_fun(*this, &Logic::onPublishStateEvent));
   event_handler->playDtmf.connect(mem_fun(*this, &Logic::playDtmf));
   event_handler->injectDtmf.connect(mem_fun(*this, &Logic::injectDtmf));
+  event_handler->setConfigValue.connect(
+          sigc::mem_fun(cfg(), &Async::Config::setValue<std::string>));
   event_handler->setVariable("mycall", m_callsign);
   char str[256];
   sprintf(str, "%.1f", report_ctcss);
@@ -656,6 +678,55 @@ bool Logic::initialize(void)
   exec_cmd_on_sql_close_timer.expired.connect(sigc::hide(
       mem_fun(dtmf_digit_handler, &DtmfDigitHandler::forceCommandComplete)));
 
+  int ctcss_to_tg_delay = 1000;
+  cfg().getValue(name(), "CTCSS_TO_TG_DELAY", ctcss_to_tg_delay);
+  m_ctcss_to_tg_timer.setTimeout(ctcss_to_tg_delay);
+  m_ctcss_to_tg_timer.expired.connect([&](Async::Timer* t)
+      {
+        //std::cout << "### ctcss_to_tg_timer expired: m_ctcss_to_tg_last_fq="
+        //          << m_ctcss_to_tg_last_fq << std::endl;
+        t->setEnable(false);
+        uint16_t uint_fq = static_cast<uint16_t>(
+            round(10.0f*m_ctcss_to_tg_last_fq));
+        auto it = m_ctcss_to_tg.find(uint_fq);
+        if (it != m_ctcss_to_tg.end())
+        {
+          uint32_t tg = it->second;
+          //cout << "### Map CTCSS " << m_ctcss_to_tg_last_fq << " to TG #"
+          //     << tg << endl;
+          setReceivedTg(tg);
+        }
+        m_ctcss_to_tg_last_fq = 0.0f;
+      });
+
+  typedef std::vector<SvxLink::SepPair<float, uint32_t> > CtcssToTgVec;
+  CtcssToTgVec ctcss_to_tg;
+  if (!cfg().getValue(name(), "CTCSS_TO_TG", ctcss_to_tg, true))
+  {
+    cerr << "*** ERROR: Illegal value for configuration variable "
+         << name() << "/CTCSS_TO_TG" << endl;
+    return false;
+  }
+  for (CtcssToTgVec::const_iterator it = ctcss_to_tg.begin();
+       it != ctcss_to_tg.end(); ++it)
+  {
+    uint16_t uint_fq = static_cast<uint16_t>(round(10.0f*it->first));
+    uint32_t tg = it->second;
+    m_ctcss_to_tg[uint_fq] = tg;
+    //if (it->first > 300)
+    //{
+    //  if (!rx().addToneDetector(it->first, 25, 10, 1000))
+    //  {
+    //    cerr << "*** WARNING: Could not setup tone detector for CTCSS "
+    //         << it->first << " to TG #" << it->second << " map in logic "
+    //         << name() << endl;
+    //  }
+    //}
+  }
+  rx().toneDetected.connect(mem_fun(*this, &Logic::detectedTone));
+
+  cfg().valueUpdated.connect(sigc::mem_fun(*this, &Logic::cfgUpdated));
+
   return true;
 
 } /* Logic::initialize */
@@ -903,7 +974,15 @@ Async::AudioSink *Logic::logicConIn(void)
 
 void Logic::setOnline(bool online)
 {
+  if (online == is_online)
+  {
+    return;
+  }
+
   is_online = online;
+  cfg().setValue(name(), "ONLINE", is_online ? "1" : "0");
+  std::cout << name() << ": Setting logic "
+            << (online ? "ONLINE" : "OFFLINE") << std::endl;
   if (online)
   {
     tx().setTxCtrlMode(currently_set_tx_ctrl_mode);
@@ -926,6 +1005,25 @@ Async::AudioSource *Logic::logicConOut(void)
 } /* Logic::logicConOut */
 
 
+void Logic::remoteCmdReceived(LogicBase* src_logic, const std::string& cmd)
+{
+  stringstream ss;
+  ss << "remote_cmd_received ";
+  ss << src_logic->name() << " ";
+  ss << cmd;
+  processEvent(ss.str());
+} /* Logic::remoteCmdReceived */
+
+
+void Logic::remoteReceivedTgUpdated(LogicBase *src_logic, uint32_t tg)
+{
+  stringstream ss;
+  ss << "remote_received_tg_updated ";
+  ss << src_logic->name() << " ";
+  ss << tg;
+  processEvent(ss.str());
+} /* Logic::remoteReceivedTgUpdated */
+
 
 /****************************************************************************
  *
@@ -933,8 +1031,19 @@ Async::AudioSource *Logic::logicConOut(void)
  *
  ****************************************************************************/
 
+Logic::~Logic(void)
+{
+  cleanup();
+  delete logic_con_out;
+  delete logic_con_in;
+  delete dtmf_digit_handler;
+} /* Logic::~Logic */
+
+
 void Logic::squelchOpen(bool is_open)
 {
+  //std::cout << "### Logic::squelchOpen: is_open=" << is_open << std::endl;
+
   if (active_module != 0)
   {
     active_module->squelchOpen(is_open);
@@ -960,6 +1069,7 @@ void Logic::squelchOpen(bool is_open)
       }
     }
     processCommandQueue();
+    setReceivedTg(0);
   }
   else
   {
@@ -1020,6 +1130,64 @@ void Logic::dtmfCtrlPtyCmdReceived(const void *buf, size_t count)
 } /* Logic::dtmfCtrlPtyCmdReceived */
 
 
+void Logic::commandPtyCmdReceived(const void *buf, size_t count)
+{
+  const char* ptr = reinterpret_cast<const char*>(buf);
+  const std::string cmdline(ptr, ptr + count);
+  //std::cout << "### Logic::commandPtyCmdReceived: " << cmdline << std::endl;
+  std::istringstream ss(cmdline);
+  std::string cmd;
+  if (!(ss >> cmd))
+  {
+    std::cerr << "*** ERROR: Invalid PTY command in logic " << name() << ": \""
+              << cmdline << "\"" << std::endl;
+    return;
+  }
+  if (cmd == "CFG")
+  {
+    std::string section, tag, value;
+    if (!(ss >> section >> tag >> value) || !ss.eof())
+    {
+      std::cerr << "*** ERROR: Invalid PTY command in logic "
+                << name() << ": \"" << cmdline << "\". "
+                << "Usage: CFG <section> <tag> <value>"
+                << std::endl;
+      return;
+    }
+    cfg().setValue(section, tag, value);
+  }
+  else if (cmd == "EVENT")
+  {
+    std::string event(std::istreambuf_iterator<char>(ss >> std::ws), {});
+    if (!ss || (event.size() == 0))
+    {
+      std::cerr << "*** ERROR: Invalid PTY command in logic "
+                << name() << ": \"" << cmdline << "\". "
+                << "Usage: EVENT <event name> [<arg> ...]"
+                << std::endl;
+      return;
+    }
+    if (event.find("::") != event.npos)
+    {
+      msg_handler->begin();
+      event_handler->processEvent(event);
+      msg_handler->end();
+    }
+    else
+    {
+      processEvent(event);
+    }
+  }
+  else
+  {
+    std::cerr << "*** ERROR: Unknown PTY command in logic "
+              << name() << ": \"" << cmdline << "\". "
+              << "Valid commands are: CFG, EVENT"
+              << std::endl;
+  }
+} /* Logic::commandPtyCmdReceived */
+
+
 void Logic::clearPendingSamples(void)
 {
   msg_handler->clear();
@@ -1154,8 +1322,8 @@ void Logic::loadModules(void)
 
 void Logic::loadModule(const string& module_cfg_name)
 {
-  cout << "Loading module \"" << module_cfg_name << "\" into logic \""
-       << name() << "\"\n";
+  std::cout << name() << ": Loading module \"" << module_cfg_name << "\""
+            << std::endl;
 
   string module_path;
   cfg().getValue("GLOBAL", "MODULE_PATH", module_path);
@@ -1441,23 +1609,16 @@ void Logic::processMacroCmd(const string& macro_cmd)
 } /* Logic::processMacroCmd */
 
 
-void Logic::checkIfOnlineCmd(void)
-{
-  if (dtmf_digit_handler->command() == (online_cmd + "1"))
-  {
-    cout << name() << ": Setting logic online\n";
-    setOnline(true);
-  }
-} /* Logic::checkIfOnlineCmd */
-
-
 void Logic::putCmdOnQueue(void)
 {
   exec_cmd_on_sql_close_timer.setEnable(false);
 
   if (!is_online)
   {
-    checkIfOnlineCmd();
+    if (dtmf_digit_handler->command() == (online_cmd + "1"))
+    {
+      setOnline(true);
+    }
     return;
   }
 
@@ -1486,10 +1647,11 @@ void Logic::timeoutNextMinute(void)
 {
   struct timeval tv;
   gettimeofday(&tv, NULL);
-  struct tm *tm = localtime(&tv.tv_sec);
-  tm->tm_min += 1;
-  tm->tm_sec = 0;
-  every_minute_timer.setTimeout(*tm);
+  struct tm tm;
+  localtime_r(&tv.tv_sec, &tm);
+  tm.tm_min += 1;
+  tm.tm_sec = 0;
+  every_minute_timer.setTimeout(tm);
 } /* Logic::timeoutNextMinute */
 
 
@@ -1504,9 +1666,10 @@ void Logic::timeoutNextSecond(void)
 {
   struct timeval tv;
   gettimeofday(&tv, NULL);
-  struct tm *tm = localtime(&tv.tv_sec);
-  tm->tm_sec += 1;
-  every_second_timer.setTimeout(*tm);
+  struct tm tm;
+  localtime_r(&tv.tv_sec, &tm);
+  tm.tm_sec += 1;
+  every_second_timer.setTimeout(tm);
 } /* Logic::timeoutNextSecond */
 
 
@@ -1576,10 +1739,10 @@ void Logic::cleanup(void)
   rgr_sound_timer.setEnable(false);
   every_minute_timer.stop();
 
-  if (LinkManager::hasInstance())
-  {
-    LinkManager::instance()->deleteLogic(this);
-  }
+  //if (LinkManager::hasInstance())
+  //{
+  //  LinkManager::instance()->deleteLogic(this);
+  //}
 
   delete event_handler;               event_handler = 0;
   delete m_tx;        	      	      m_tx = 0;
@@ -1592,6 +1755,7 @@ void Logic::cleanup(void)
   delete qso_recorder;                qso_recorder = 0;
   delete state_pty;                   state_pty = 0;
   delete dtmf_ctrl_pty;               dtmf_ctrl_pty = 0;
+  delete command_pty;                 command_pty = 0;
 } /* Logic::cleanup */
 
 
@@ -1623,8 +1787,10 @@ void Logic::audioFromModuleStreamStateChanged(bool is_active, bool is_idle)
 } /* Logic::audioFromModuleStreamStateChanged */
 
 
-void Logic::publishStateEvent(const string &event_name, const string &msg)
+void Logic::onPublishStateEvent(const string &event_name, const string &msg)
 {
+  publishStateEvent(event_name, msg);
+
   if (state_pty == 0)
   {
     return;
@@ -1637,7 +1803,38 @@ void Logic::publishStateEvent(const string &event_name, const string &msg)
   os << event_name << " " << msg;
   os << endl;
   state_pty->write(os.str().c_str(), os.str().size());
-} /* Logic::publishStateEvent */
+} /* Logic::onPublishStateEvent */
+
+
+void Logic::detectedTone(float fq)
+{
+  //std::cout << "### Logic::detectedTone[" << name() << "]: " << fq
+  //          << " Hz tone call detected" << endl;
+  m_ctcss_to_tg_last_fq = fq;
+  m_ctcss_to_tg_timer.setEnable(true);
+} /* Logic::detectedTone */
+
+
+void Logic::cfgUpdated(const std::string& section, const std::string& tag)
+{
+  if (section == name())
+  {
+    std::string value;
+    if (cfg().getValue(name(), tag, value))
+    {
+      event_handler->setVariable("Logic::CFG_" + tag, value);
+      processEvent("config_updated CFG_" + tag + " \"" + value + "\"");
+    }
+    if (tag == "ONLINE")
+    {
+      bool online;
+      if (cfg().getValue(name(), "ONLINE", online))
+      {
+        setOnline(online);
+      }
+    }
+  }
+} /* Logic::cfgUpdated */
 
 
 /*
