@@ -8,7 +8,7 @@ This file contains a class that implements a local transmitter.
 
 \verbatim
 SvxLink - A Multi Purpose Voice Services System for Ham Radio Use
-Copyright (C) 2003-2019 Tobias Blomberg / SM0SVX
+Copyright (C) 2003-2023 Tobias Blomberg / SM0SVX
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -112,7 +112,8 @@ using namespace SvxLink;
  *
  ****************************************************************************/
 
-#define USE_AUDIO_VALVE 0
+#define USE_AUDIO_VALVE         0
+#define DEFAULT_LIMITER_THRESH  0.0
 
 
 /****************************************************************************
@@ -126,7 +127,7 @@ class SineGenerator : public Async::AudioSource
   public:
     explicit SineGenerator(const string& audio_dev, int channel)
       : audio_io(audio_dev, channel), pos(0), fq(0.0), level(0.0),
-        sample_rate(0), audio_dev_keep_open(false)
+        sample_rate(0)
     {
       sample_rate = audio_io.sampleRate();
       audio_io.registerSource(this);
@@ -142,16 +143,11 @@ class SineGenerator : public Async::AudioSource
       fq = tone_fq;
     }
     
-    void setLevel(int level_percent)
+    void setLevel(float level_db)
     {
-      level = level_percent / 100.0;
+      level = powf(10.0f, level_db / 20.0f);
     }
 
-    void setKeepOpen(bool keep_open)
-    {
-      audio_dev_keep_open = keep_open;
-    }
-    
     void enable(bool enable)
     {
       if (enable == (audio_io.mode() != AudioIO::MODE_NONE))
@@ -167,7 +163,7 @@ class SineGenerator : public Async::AudioSource
           writeSamples();
         }
       }
-      else if (!audio_dev_keep_open)
+      else
       {
       	audio_io.close();
       }
@@ -194,7 +190,6 @@ class SineGenerator : public Async::AudioSource
     double    fq;
     double    level;
     int       sample_rate;
-    bool      audio_dev_keep_open;
     
     void writeSamples(void)
     {
@@ -286,7 +281,13 @@ LocalTx::~LocalTx(void)
 bool LocalTx::initialize(void)
 {
   string value;
-  
+
+  char tx_id = '\0';
+  if (cfg.getValue(name(), "TX_ID", tx_id))
+  {
+    setId(tx_id);
+  }
+
   string audio_dev;
   if (!cfg.getValue(name(), "AUDIO_DEV", audio_dev))
   {
@@ -381,18 +382,29 @@ bool LocalTx::initialize(void)
   }
 
   sine_gen = new SineGenerator(audio_dev, audio_channel);
-  sine_gen->setKeepOpen(audio_dev_keep_open);
-  
+
   if (cfg.getValue(name(), "CTCSS_FQ", value))
   {
     sine_gen->setFq(atof(value.c_str()));
-  }  
-  
-  if (cfg.getValue(name(), "CTCSS_LEVEL", value))
+  }
+
+  float ctcss_level = 0.0f;
+  if (cfg.getValue(name(), "CTCSS_LEVEL", ctcss_level))
   {
-    int level = atoi(value.c_str());
-    sine_gen->setLevel(level);
-    //audio_io->setGain((100.0 - level) / 100.0);
+    if (ctcss_level <= 0.0f)
+    {
+      sine_gen->setLevel(ctcss_level);
+    }
+    else
+    {
+      float ctcss_level_db = 20.0f*log10f(ctcss_level / 100.0f);
+      sine_gen->setLevel(ctcss_level_db);
+      std::cerr << "*** WARNING: Setting " << name()
+                << "/CTCSS_LEVEL in percent is deprecated. It should be set "
+                   "in dBFS instead. To get about the same level as "
+                   "configured now, set CTCSS_LEVEL="
+                << round(ctcss_level_db) << "." << std::endl;
+    }
   }
 
 #if INTERNAL_SAMPLE_RATE >= 16000
@@ -465,24 +477,28 @@ bool LocalTx::initialize(void)
     prev_src->registerSink(preemph, true);
     prev_src = preemph;
   }
-  
-  /*
-  AudioCompressor *limit = new AudioCompressor;
-  limit->setThreshold(-1);
-  limit->setRatio(0.1);
-  limit->setAttack(2);
-  limit->setDecay(20);
-  limit->setOutputGain(1);
-  prev_src->registerSink(limit, true);
-  prev_src = limit;
-  */
-  
+
+    // Add a limiter to smoothly limit the audio before hard clipping it
+  double limiter_thresh = DEFAULT_LIMITER_THRESH;
+  cfg.getValue(name(), "LIMITER_THRESH", limiter_thresh);
+  if (limiter_thresh != 0.0)
+  {
+    AudioCompressor *limit = new AudioCompressor;
+    limit->setThreshold(limiter_thresh);
+    limit->setRatio(0.1);
+    limit->setAttack(2);
+    limit->setDecay(20);
+    limit->setOutputGain(1);
+    prev_src->registerSink(limit, true);
+    prev_src = limit;
+  }
+
     // Clip audio to limit its amplitude
   AudioClipper *clipper = new AudioClipper;
   prev_src->registerSink(clipper, true);
   prev_src = clipper;
   
-#if 1
+#if 0
     // Filter out high frequencies generated by the previous clipping
 #if (INTERNAL_SAMPLE_RATE == 16000)
   //AudioFilter *splatter_filter = new AudioFilter("LpBu10/5500");
@@ -493,7 +509,17 @@ bool LocalTx::initialize(void)
   prev_src->registerSink(splatter_filter, true);
   prev_src = splatter_filter;
 #endif
-  
+
+#if (INTERNAL_SAMPLE_RATE == 16000)
+  AudioFilter *voiceband_filter =
+    new AudioFilter("LpCh9/-0.05/5500 x HpCh12/-0.05/300");
+#else
+  AudioFilter *voiceband_filter =
+    new AudioFilter("LpBu20/3500 x HpCh12/-0.05/300");
+#endif
+  prev_src->registerSink(voiceband_filter, true);
+  prev_src = voiceband_filter;
+
     // Create a valve so that we can control when to transmit audio
   #if USE_AUDIO_VALVE
   audio_valve = new AudioValve;
@@ -697,6 +723,17 @@ bool LocalTx::initialize(void)
     }
   }
 
+  if (!audio_io->open(AudioIO::MODE_WR))
+  {
+    cerr << "*** ERROR: Could not open audio device for transmitter \""
+         << name() << "\"\n";
+    return false;
+  }
+
+  if (!audio_dev_keep_open)
+  {
+    audio_io->close();
+  }
   return true;
   
 } /* LocalTx::initialize */
