@@ -6,7 +6,7 @@
 
 \verbatim
 SvxLink - A Multi Purpose Voice Services System for Ham Radio Use
-Copyright (C) 2003-2014 Tobias Blomberg / SM0SVX
+Copyright (C) 2003-2020 Tobias Blomberg / SM0SVX
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ****************************************************************************/
 
 #include <iostream>
+#include <cassert>
 
 
 /****************************************************************************
@@ -57,6 +58,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "SigLevDetDdr.h"
 #include "SigLevDetSim.h"
 #include "SigLevDetAfsk.h"
+#include "SigLevDetConst.h"
 
 
 
@@ -92,10 +94,8 @@ namespace {
   class SigLevDetNone : public SigLevDet
   {
     public:
-      struct Factory : public SigLevDetFactory<SigLevDetNone>
-      {
-        Factory(void) : SigLevDetFactory<SigLevDetNone>("NONE") {}
-      };
+      /// The name of this class when used by the object factory
+    static constexpr const char* OBJNAME = "NONE";
 
     virtual void setContinuousUpdateInterval(int interval_ms) {}
     virtual void setIntegrationTime(int time_ms) {}
@@ -139,32 +139,51 @@ namespace {
  *
  ****************************************************************************/
 
-SigLevDet *SigLevDetFactoryBase::createNamedSigLevDet(Config& cfg,
-                                                      const string& name)
+SigLevDet* createSigLevDet(Async::Config& cfg, const std::string& name)
 {
-  SigLevDetNone::Factory none_siglev_factory;
-  SigLevDetNoise::Factory noise_siglev_factory;
-  SigLevDetTone::Factory tone_siglev_factory;
-  SigLevDetDdr::Factory ddr_siglev_factory;
-  SigLevDetSim::Factory sim_siglev_factory;
-  SigLevDetAfsk::Factory afsk_siglev_factory;
-  
+  static SigLevDetSpecificFactory<SigLevDetNone> none_siglev_factory;
+  static SigLevDetSpecificFactory<SigLevDetNoise> noise_siglev_factory;
+  static SigLevDetSpecificFactory<SigLevDetTone> tone_siglev_factory;
+  static SigLevDetSpecificFactory<SigLevDetDdr> ddr_siglev_factory;
+  static SigLevDetSpecificFactory<SigLevDetSim> sim_siglev_factory;
+  static SigLevDetSpecificFactory<SigLevDetAfsk> afsk_siglev_factory;
+  static SigLevDetSpecificFactory<SigLevDetConst> const_siglev_factory;
+
   string det_name;
   if (!cfg.getValue(name, "SIGLEV_DET", det_name) || det_name.empty())
   {
     det_name = "NOISE";
   }
 
-  SigLevDet *det = createNamedObject(det_name);
+  auto& det_map = SigLevDet::detMap();
+  auto it = det_map.find(name);
+  if (it != det_map.end())
+  {
+    return it->second;
+  }
+
+  SigLevDet *det = SigLevDetFactory::createNamedObject(det_name);
   if (det == 0)
   {
     cerr << "*** ERROR: Unknown SIGLEV_DET \"" << det_name
          << "\" specified for receiver " << name << ". Legal values are: "
-         << validFactories() << endl;
+         << SigLevDetFactory::validFactories() << endl;
+    return 0;
   }
-  
+
+  if (!det->initialize(cfg, name, INTERNAL_SAMPLE_RATE))
+  {
+    cout << "*** ERROR: Could not initialize the signal level detector for "
+         << "receiver " << name << std::endl;
+    delete det;
+    det = 0;
+    return 0;
+  }
+
+  det_map[name] = det;
+
   return det;
-} /* SigLevDetFactoryBase::createNamedSigLevDet */
+} /* createSigLevDet */
 
 
 SigLevDet::SigLevDet(void)
@@ -173,16 +192,20 @@ SigLevDet::SigLevDet(void)
 } /* SigLevDet::SigLevDet */
 
 
-#if 0
 SigLevDet::~SigLevDet(void)
 {
+  auto it = detMap().find(name);
+  if (it != detMap().end())
+  {
+    detMap().erase(it);
+  }
 } /* SigLevDet::~SigLevDet */
-#endif
 
 
 bool SigLevDet::initialize(Async::Config &cfg, const std::string& name,
                            int sample_rate)
 {
+  this->name = name;
   if (cfg.getValue(name, "RX_ID", last_rx_id))
   {
     force_rx_id = true;
