@@ -1,14 +1,14 @@
 /**
-@file   Template.cpp
+@file   MyNamespaceTemplate.cpp
 @brief  A_brief_description_for_this_file
 @author Tobias Blomberg / SM0SVX
-@date   2019-
+@date   2024-
 
 A_detailed_description_for_this_file
 
 \verbatim
 <A brief description of the program or library this file belongs to>
-Copyright (C) 2003-2019 Tobias Blomberg / SM0SVX
+Copyright (C) 2003-2024 Tobias Blomberg / SM0SVX
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -26,8 +26,6 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 \endverbatim
 */
 
-
-
 /****************************************************************************
  *
  * System Includes
@@ -50,8 +48,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  ****************************************************************************/
 
-#include "Template.h"
-
+#include "MyNamespaceTemplate.h"
 
 
 /****************************************************************************
@@ -60,8 +57,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  ****************************************************************************/
 
-using namespace std;
-
+using namespace MyNamespace;
 
 
 /****************************************************************************
@@ -72,38 +68,32 @@ using namespace std;
 
 
 
+/****************************************************************************
+ *
+ * Static class variables
+ *
+ ****************************************************************************/
+
+
+
 /****************************************************************************
  *
  * Local class definitions
  *
  ****************************************************************************/
 
+namespace {
 
 
 /****************************************************************************
  *
- * Prototypes
+ * Local functions
  *
  ****************************************************************************/
 
 
 
-/****************************************************************************
- *
- * Exported Global Variables
- *
- ****************************************************************************/
-
-
-
-
-/****************************************************************************
- *
- * Local Global Variables
- *
- ****************************************************************************/
-
-
+}; /* End of anonymous namespace */
 
 /****************************************************************************
  *
@@ -113,11 +103,13 @@ using namespace std;
 
 Template::Template(void)
 {
+
 } /* Template::Template */
 
 
 Template::~Template(void)
 {
+
 } /* Template::~Template */
 
 
@@ -135,6 +127,8 @@ Template::~Template(void)
  *
  ****************************************************************************/
 
+
+
 /*
  * This file has not been truncated
  */
